@@ -16,7 +16,7 @@ var appletMeta;
 // Maps uuid -> importer object (applet directory tree)
 var applets;
 // Maps applet_id -> applet objects
-const appletObj = {};
+const appletObj = [];
 var appletsLoaded = false;
 
 // An applet can assume a role
@@ -28,7 +28,6 @@ var appletsLoaded = false;
 // the same role?
 const Roles = {
     NOTIFICATIONS: 'notifications',
-    WINDOWLIST: 'windowlist',
     PANEL_LAUNCHER: 'panellauncher'
 }
 
@@ -36,8 +35,8 @@ let enabledAppletDefinitions;
 let clipboard = [];
 
 function init() {
-    appletMeta = Extension.meta;
-    applets = Extension.importObjects;
+    applets = Extension.Type.APPLET.maps.importObjects;
+    appletMeta = Extension.Type.APPLET.maps.meta;
 
     appletsLoaded = false;
     
@@ -65,10 +64,10 @@ function finishExtensionLoad(extension) {
 }
 
 // Callback for extension.js
-function prepareExtensionUnload(extension) {
+function prepareExtensionUnload(extension, deleteConfig) {
     // Remove all applet instances for this extension
     for(let applet_id in extension._loadedDefinitions) {
-        removeAppletFromPanels(extension._loadedDefinitions[applet_id]);
+        removeAppletFromPanels(extension._loadedDefinitions[applet_id], deleteConfig);
     }
 }
 
@@ -79,7 +78,7 @@ function getEnabledAppletDefinitions() {
         // maps uuid -> list of applet definitions
         uuidMap: {},
         // maps applet_id -> single applet definition
-        idMap: {}
+        idMap: []
     };
     
     // Upgrade settings if required
@@ -116,17 +115,8 @@ function getAppletDefinition(definition) {
         // Panel might not exist. Still keep definition for future use.
         let location;
         if (panel) {
-            orientation = panel.bottomPosition ? St.Side.BOTTOM : St.Side.TOP;
-            switch (elements[1]){
-            case "center":
-                location = panel._centerBox;
-                break;
-            case "right":
-                location = panel._rightBox;
-                break;
-            default: // Let default position be left
-                location = panel._leftBox;
-            }
+            orientation = setOrientationForPanel(panel.panelPosition);
+            location = getLocation(panel, elements[1]);
         }
         
         return {
@@ -146,6 +136,43 @@ function getAppletDefinition(definition) {
     return null;
 }
 
+function setOrientationForPanel(panelPos) {
+    let orientation;
+    switch (panelPos)
+    {
+        case 0:
+                orientation = St.Side.TOP;
+        break;
+        case 1:
+                orientation = St.Side.BOTTOM;
+        break;
+        case 2:
+                orientation = St.Side.LEFT;
+        break;
+        case 3:
+                orientation = St.Side.RIGHT;
+        break;
+    }
+
+    return orientation;
+}
+
+function setHeightForPanel(panelObj, panelPos) {
+    let height;
+    switch (panelPos)  // for vertical panels use the width instead of the height
+    {
+        case 0:
+        case 1:
+                height = panelObj.actor.get_height();
+        break;
+        case 2:
+        case 3:
+                height = panelObj.actor.get_width();
+        break;
+    }
+    return height;
+}
+
 function checkForUpgrade(newEnabledApplets) {
     // upgrade if old version
     let nextAppletId = global.settings.get_int("next-applet-id");
@@ -177,14 +204,15 @@ function onEnabledAppletsChanged() {
         // Remove all applet instances that do not exist in the definition anymore.
         for (let applet_id in oldEnabledAppletDefinitions.idMap) {
             if(!enabledAppletDefinitions.idMap[applet_id]) {
-                removeAppletFromPanels(oldEnabledAppletDefinitions.idMap[applet_id]);
+                let removeConfig = Extension.get_max_instances(oldEnabledAppletDefinitions.idMap[applet_id].uuid, Extension.Type.APPLET) != 1
+                removeAppletFromPanels(oldEnabledAppletDefinitions.idMap[applet_id], removeConfig);
             }
         }
         
         // Unload all applet extensions that do not exist in the definition anymore.
         for (let uuid in oldEnabledAppletDefinitions.uuidMap) {
             if(!enabledAppletDefinitions.uuidMap[uuid]) {
-                Extension.unloadExtension(uuid);
+                Extension.unloadExtension(uuid, Extension.Type.APPLET);
             }
         }
         
@@ -194,7 +222,7 @@ function onEnabledAppletsChanged() {
             let oldDef = oldEnabledAppletDefinitions.idMap[applet_id];
             
             if(!oldDef || !appletDefinitionsEqual(newDef, oldDef)) {
-                let extension = Extension.objects[newDef.uuid];
+                let extension = Extension.Type.APPLET.maps.objects[newDef.uuid];
                 if(extension) {
                     addAppletToPanels(extension, newDef);
                 }
@@ -214,7 +242,7 @@ function onEnabledAppletsChanged() {
     Main.statusIconDispatcher.redisplay();
 }
 
-function removeAppletFromPanels(appletDefinition) {
+function removeAppletFromPanels(appletDefinition, deleteConfig) {
     let applet = appletObj[appletDefinition.applet_id];
     if (applet) {
         try {
@@ -231,7 +259,8 @@ function removeAppletFromPanels(appletDefinition) {
         delete applet._extension._loadedDefinitions[appletDefinition.applet_id];
         delete appletObj[appletDefinition.applet_id];
 
-        _removeAppletConfigFile(appletDefinition.uuid, appletDefinition.applet_id);
+        if (deleteConfig)
+            _removeAppletConfigFile(appletDefinition.uuid, appletDefinition.applet_id);
 
         /* normal occurs during _onAppletRemovedFromPanel, but when a panel is removed,
          * appletObj hasn't had the instance removed yet, so let's run it one more time
@@ -265,7 +294,7 @@ function addAppletToPanels(extension, appletDefinition) {
         let applet = createApplet(extension, appletDefinition);
         if(applet == null)
             return false;
-        
+
         // Now actually lock the applets role and set the provider
         extension.lockRole(applet);
 
@@ -278,31 +307,19 @@ function addAppletToPanels(extension, appletDefinition) {
             applet._panelLocation = null;
         }
 
-        // Add it to its new panel location
-        let children = appletDefinition.location.get_children();
-        let appletsToMove = [];
-        for (let i=0; i<children.length;i++) {
-            let child = children[i];
-            if ((typeof child._applet !== "undefined") && (child._applet instanceof Applet.Applet)) {
-                if (appletDefinition.order < child._applet._order) {
-                    appletsToMove.push(child);
-                }
-            }
-        }
+        let location = appletDefinition.location;
 
-        for (let i=0; i<appletsToMove.length; i++) {
-            appletDefinition.location.remove_actor(appletsToMove[i]);
-        }
+        let before = location.get_children()
+            .find(x => (x._applet instanceof Applet.Applet) &&
+                       (appletDefinition.order < x._applet._order));
 
-        appletDefinition.location.add(applet.actor);
+        if (before)
+            location.insert_child_below(applet.actor, before);
+        else
+            location.add_actor(applet.actor);
+
+        applet._panelLocation = location;
 
-        applet._panelLocation = appletDefinition.location;
-        for (let i=0; i<appletsToMove.length; i++) {
-            let hidden = !appletsToMove[i].visible;
-            appletDefinition.location.add(appletsToMove[i]);
-            if (hidden) appletsToMove[i].hide();
-        }
-        
         if(!extension._loadedDefinitions) {
             extension._loadedDefinitions = {};
         }
@@ -310,15 +327,48 @@ function addAppletToPanels(extension, appletDefinition) {
 
         applet.on_applet_added_to_panel_internal(appletsLoaded);
 
+        removeAppletFromInappropriatePanel (extension, applet, appletDefinition);
+
         return true;
-    }
-    catch(e) {
+    } catch(e) {
         extension.unlockRole();
         extension.logError('Failed to load applet: ' + appletDefinition.uuid + "/" + appletDefinition.applet_id, e);
         return false;
     }
 }
 
+function removeAppletFromInappropriatePanel (extension, applet, appletDefinition) {
+    //  We want to ensure that applets placed in a panel can be shown correctly
+    //  - particularly because wide applets will not fit in a vertical panel unless
+    //  they have logic to manage this explicitly
+    //  If the applet is of type Icon Applet (and not a text icon applet) then should be fine otherwise
+    //  we look to see if it has declared itself suitable via a getAllowedLayout call
+    //
+    //  If the applet turns out to be unsuitable then remove it.  The applet will show with a red
+    //  indicator in the applet list
+    if (applet instanceof Applet.IconApplet && !(applet instanceof Applet.TextIconApplet)) {
+        ;
+    } else {
+        let allowedLayout = applet.getAllowedLayout();
+
+        if ((appletDefinition.orientation == St.Side.LEFT || appletDefinition.orientation == St.Side.RIGHT) &&
+             allowedLayout == Applet.AllowedLayout.HORIZONTAL) {
+            global.logError("applet " + appletDefinition.uuid + " not suitable for panel orientation " + appletDefinition.orientation);
+            removeAppletFromPanels(extension._loadedDefinitions[appletDefinition.applet_id]);
+            let dialog = new ModalDialog.NotifyDialog(_("This applet is not suitable for vertical panels") + "\n\n");
+            dialog.open();
+        } else if ((appletDefinition.orientation == St.Side.TOP || appletDefinition.orientation == St.Side.BOTTOM) &&
+                    allowedLayout == Applet.AllowedLayout.VERTICAL) {
+                global.logError("applet " + appletDefinition.uuid + " not suitable for panel orientation " + appletDefinition.orientation);
+                removeAppletFromPanels(extension._loadedDefinitions[appletDefinition.applet_id]);
+                let dialog = new ModalDialog.NotifyDialog(_("This applet is not suitable for horizontal panels") + "\n\n");
+                dialog.open();
+        }
+    }
+}
+
+
+
 function get_role_provider(role) {
     if (Extension.Type.APPLET.roles[role]) {
         return Extension.Type.APPLET.roles[role].roleProvider;
@@ -335,14 +385,17 @@ function createApplet(extension, appletDefinition) {
 
     let applet_id = appletDefinition.applet_id;
     let orientation = appletDefinition.orientation;
-    let panel_height =  appletDefinition.panel.actor.get_height();
+    let panel_height;
+
+    panel_height = setHeightForPanel(appletDefinition.panel, appletDefinition.panel.panelPosition)
     
     if (appletObj[applet_id] != undefined) {
         global.log(applet_id + ' applet already loaded');
+        appletObj[applet_id].setOrientation(orientation);
         if (appletObj[applet_id]._panelHeight != panel_height) {
             appletObj[applet_id].setPanelHeight(panel_height);
         }
-        appletObj[applet_id].setOrientation(orientation);
+
         return appletObj[applet_id];
     }
     
@@ -436,7 +489,9 @@ function updateAppletPanelHeights(force_recalc) {
     for (let applet_id in enabledAppletDefinitions.idMap) {
         if (appletObj[applet_id]) {
             let appletDefinition = enabledAppletDefinitions.idMap[applet_id];
-            let newheight = appletDefinition.panel.actor.get_height();
+            let newheight;
+            newheight = setHeightForPanel(appletDefinition.panel, appletDefinition.panel.panelPosition);
+
             if (appletObj[applet_id]._panelHeight != newheight || force_recalc) {
                 appletObj[applet_id].setPanelHeight(newheight);
             }
@@ -457,13 +512,9 @@ function get_object_for_instance (appletId) {
     }
 }
 
-function get_object_for_uuid (uuid) {
-    for (let instanceid in appletObj) {
-        if (appletObj[instanceid]._uuid == uuid) {
-            return appletObj[instanceid]
-        }
-    }
-    return null;
+function get_object_for_uuid (uuid, instanceId) {
+    return appletObj.find(x => x && x._uuid == uuid &&
+                               (x.instance_id == instanceId || instanceId == uuid));
 }
 
 
@@ -474,30 +525,20 @@ function get_object_for_uuid (uuid) {
  * Loads all applets on the panel if not loaded
  */
 function loadAppletsOnPanel(panel) {
-    let orientation = panel.bottomPosition ? St.Side.BOTTOM : St.Side.TOP;
+    let orientation;
+
+    orientation = setOrientationForPanel(panel.panelPosition);
+
     let definition;
 
     for (let applet_id in enabledAppletDefinitions.idMap){
         definition = enabledAppletDefinitions.idMap[applet_id];
         if(definition.panelId == panel.panelId) {
-            let location;
-            // Update appletDefinition
-            switch (definition.location_label){
-            case "center":
-                location = panel._centerBox;
-                break;
-            case "right":
-                location = panel._rightBox;
-                break;
-            default: // Let default position be left
-                location = panel._leftBox;
-            }
-
             definition.panel = panel;
-            definition.location = location;
+            definition.location = getLocation(panel, definition.location_label);
             definition.orientation = orientation;
 
-            let extension = Extension.objects[definition.uuid];
+            let extension = Extension.Type.APPLET.maps.objects[definition.uuid];
             if(extension) {
                 addAppletToPanels(extension, definition);
             }
@@ -512,35 +553,28 @@ function loadAppletsOnPanel(panel) {
  * Updates the definition, orientation and height of applets on the panel
  */
 function updateAppletsOnPanel (panel) {
-    let height = panel.actor.get_height();
-    let orientation = panel.bottomPosition ? St.Side.BOTTOM : St.Side.TOP;
+    let height;
+    let orientation;
+
+    orientation = setOrientationForPanel(panel.panelPosition);
+    height = setHeightForPanel(panel, panel.panelPosition);
+
     let definition;
 
     for (let applet_id in enabledAppletDefinitions.idMap){
         definition = enabledAppletDefinitions.idMap[applet_id];
         if(definition.panel == panel) {
-            let location;
-            switch (definition.location_label[1]){
-            case "center":
-                location = panel._centerBox;
-                break;
-            case "right":
-                location = panel._rightBox;
-                break;
-            default: // Let default position be left
-                location = panel._leftBox;
-            }
-
-            definition.location = location;
+            definition.location = getLocation(panel, definition.location_label);
             definition.orientation = orientation;
 
             if (appletObj[applet_id]) {
                 try {
-                    appletObj[applet_id].setPanelHeight(height);
                     appletObj[applet_id].setOrientation(orientation);
+                    appletObj[applet_id].setPanelHeight(height);
                 } catch (e) {
                     global.logError("Error during setPanelHeight() and setOrientation() call on applet: " + definition.uuid + "/" + applet_id, e);
                 }
+                removeAppletFromInappropriatePanel (Extension.Type.APPLET.maps.objects[definition.uuid], appletObj[applet_id], definition);
             }
         }
     }
@@ -553,95 +587,51 @@ function updateAppletsOnPanel (panel) {
  * Unloads all applets on the panel
  */
 function unloadAppletsOnPanel (panel) {
-    for (let applet_id in enabledAppletDefinitions.idMap){
-        if(enabledAppletDefinitions.idMap[applet_id].panel == panel) {
-            removeAppletFromPanels(enabledAppletDefinitions.idMap[applet_id]);
-        }
-    }
+    let panelApplets = enabledAppletDefinitions.idMap.filter(x => x.panel == panel);
+    for (let i in panelApplets)
+        removeAppletFromPanels(panelApplets[i], false);
 }
 
 function copyAppletConfiguration(panelId) {
-    let def = enabledAppletDefinitions.idMap;
-    clipboard = [];
-    for (let i in def) {
-        if (def[i].panelId == panelId) {
-            clipboard.push(def[i]);
-        }
-    }
+    clipboard = enabledAppletDefinitions.idMap.filter(x => x.panelId == panelId);
 }
 
 function clearAppletConfiguration(panelId) {
     let raw = global.settings.get_strv("enabled-applets");
-
-    // Remove existing applets on panel
-    let i = raw.length;
-    while(i--) { // Do a reverse loop to prevent skipping items after splicing
-        if (raw[i].split(":")[0].slice(5) == panelId)
-            raw.splice(i,1);
-    }
+    raw = raw.filter(x => x.split(":")[0].slice(5) != panelId);
     global.settings.set_strv("enabled-applets", raw);
 }
 
 function pasteAppletConfiguration(panelId) {
-    let raw = global.settings.get_strv("enabled-applets");
-
-    // Remove existing applets on panel
-    let i = raw.length;
-    while(i--) { // Do a reverse loop to prevent skipping items after splicing
-        if (raw[i].split(":")[0].slice(5) == panelId)
-            raw.splice(i,1);
-    }
+    clearAppletConfiguration(panelId);
 
     let skipped = false;
-    let len = clipboard.length;
-    let nextId = global.settings.get_int("next-applet-id");
-    for (let i = 0; i < len; i++) {
-        let max = Extension.get_max_instances(clipboard[i].uuid);
-        if (max == -1) {
-            raw.push("panel" + panelId + ":" + clipboard[i].location_label + ":" + clipboard[i].order + ":" + clipboard[i].uuid + ":" + nextId);
-            nextId ++;
-            continue;
-        }
-        let curr = enabledAppletDefinitions.uuidMap[clipboard[i].uuid];
-        let count = curr.length;
-        if (count >= max) { // If we have more applets that allowed, we see if we any of them are removed above
-            let i = count;
-            while (i--) { // Do a reverse loop because the value of count will change
-                if (curr[i].panelId == panelId) count --;
-            }
-        }
 
-        if (count < max) {
-            raw.push("panel" + panelId + ":" + clipboard[i].location_label + ":" + clipboard[i].order + ":" + clipboard[i].uuid + ":" + nextId);
+    let raw = global.settings.get_strv("enabled-applets");
+    let nextId = global.settings.get_int("next-applet-id");
+
+    clipboard.forEach(function(x) {
+        let uuid = x.uuid
+        let max = Extension.get_max_instances(uuid, Extension.Type.APPLET);
+        if (max == -1 || raw.filter(a => a.split(":")[3] == uuid).length < max) {
+            raw.push("panel" + panelId + ":" + x.location_label + ":" + x.order + ":" + uuid + ":" + nextId);
             nextId ++;
         } else {
             skipped = true;
         }
-    }
+    });
+
     global.settings.set_int("next-applet-id", nextId);
     global.settings.set_strv("enabled-applets", raw);
 
     if (skipped) {
-        let dialog = new ModalDialog.NotifyDialog(_("Certain applets do not allow multiple instances and were not copied") + "\n\n");
+        let dialog = new ModalDialog.NotifyDialog(_("Certain applets do not allow multiple instances or were at their max number of instances so were not copied") + "\n\n");
         dialog.open();
     }
 }
 
 function getRunningInstancesForUuid(uuid) {
-    if(!enabledAppletDefinitions)
-        return null;
-
-    let result = [];
-
-    for (let applet_id in enabledAppletDefinitions.idMap) {
-        if (appletObj[applet_id]) {
-            if (uuid == appletObj[applet_id]._uuid) {
-                result.push(appletObj[applet_id]);
-            }
-        }
-    }
-
-    return result
+    return appletObj.filter(x => x && x._uuid == uuid);
 }
 
 function callAppletInstancesChanged(uuid) {
@@ -654,3 +644,8 @@ function callAppletInstancesChanged(uuid) {
     }
 }
 
+function getLocation(panel, location) {
+    return {"center": panel._centerBox,
+            "right" : panel._rightBox,
+            "left"  : panel._leftBox}[location];
+}
