@@ -63,7 +63,6 @@
 #include "st-icon.h"
 #include "st-widget.h"
 #include "st-texture-cache.h"
-#include "st-marshal.h"
 #include "st-clipboard.h"
 #include "st-private.h"
 
@@ -392,13 +391,11 @@ st_entry_allocate (ClutterActor          *actor,
 {
   StEntryPrivate *priv = ST_ENTRY_PRIV (actor);
   StThemeNode *theme_node = st_widget_get_theme_node (ST_WIDGET (actor));
-  ClutterActorClass *parent_class;
   ClutterActorBox content_box, child_box, icon_box;
   gfloat icon_w, icon_h;
   gfloat entry_h, min_h, pref_h, avail_h;
 
-  parent_class = CLUTTER_ACTOR_CLASS (st_entry_parent_class);
-  parent_class->allocate (actor, box, flags);
+  clutter_actor_set_allocation (actor, box, flags);
 
   st_theme_node_get_content_box (theme_node, box, &content_box);
 
@@ -717,41 +714,6 @@ clutter_text_cursor_changed (ClutterText *text, ClutterActor *actor)
   st_entry_pend_cursor_blink (ST_ENTRY (actor));
 }
 
-static void
-st_entry_paint (ClutterActor *actor)
-{
-  StEntryPrivate *priv = ST_ENTRY_PRIV (actor);
-  ClutterActorClass *parent_class;
-
-  parent_class = CLUTTER_ACTOR_CLASS (st_entry_parent_class);
-  parent_class->paint (actor);
-
-  clutter_actor_paint (priv->entry);
-
-  if (priv->primary_icon)
-    clutter_actor_paint (priv->primary_icon);
-
-  if (priv->secondary_icon)
-    clutter_actor_paint (priv->secondary_icon);
-}
-
-static void
-st_entry_pick (ClutterActor       *actor,
-               const ClutterColor *c)
-{
-  StEntryPrivate *priv = ST_ENTRY_PRIV (actor);
-
-  CLUTTER_ACTOR_CLASS (st_entry_parent_class)->pick (actor, c);
-
-  clutter_actor_paint (priv->entry);
-
-  if (priv->primary_icon)
-    clutter_actor_paint (priv->primary_icon);
-
-  if (priv->secondary_icon)
-    clutter_actor_paint (priv->secondary_icon);
-}
-
 static void
 st_entry_clipboard_callback (StClipboard *clipboard,
                              const gchar *text,
@@ -868,8 +830,6 @@ st_entry_class_init (StEntryClass *klass)
   actor_class->get_preferred_width = st_entry_get_preferred_width;
   actor_class->get_preferred_height = st_entry_get_preferred_height;
   actor_class->allocate = st_entry_allocate;
-  actor_class->paint = st_entry_paint;
-  actor_class->pick = st_entry_pick;
 
   actor_class->key_press_event = st_entry_key_press_event;
   actor_class->key_focus_in = st_entry_key_focus_in;
@@ -879,10 +839,10 @@ st_entry_class_init (StEntryClass *klass)
   widget_class->get_accessible_type = st_entry_accessible_get_type;
 
   pspec = g_param_spec_object ("clutter-text",
-			       "Clutter Text",
-			       "Internal ClutterText actor",
-			       CLUTTER_TYPE_TEXT,
-			       G_PARAM_READABLE);
+             "Clutter Text",
+             "Internal ClutterText actor",
+             CLUTTER_TYPE_TEXT,
+             G_PARAM_READABLE);
   g_object_class_install_property (gobject_class, PROP_CLUTTER_TEXT, pspec);
 
   pspec = g_param_spec_string ("hint-text",
@@ -909,8 +869,7 @@ st_entry_class_init (StEntryClass *klass)
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (StEntryClass, primary_icon_clicked),
-                  NULL, NULL,
-                  _st_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
   /**
    * StEntry::secondary-icon-clicked:
@@ -922,8 +881,7 @@ st_entry_class_init (StEntryClass *klass)
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (StEntryClass, secondary_icon_clicked),
-                  NULL, NULL,
-                  _st_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 }
 
@@ -1138,7 +1096,7 @@ _st_entry_set_icon (StEntry       *entry,
       g_signal_handlers_disconnect_by_func (*icon,
                                             _st_entry_icon_press_cb,
                                             entry);
-      clutter_actor_unparent (*icon);
+      clutter_actor_remove_child (CLUTTER_ACTOR (entry), *icon);
       *icon = NULL;
     }
 
